from typing import List

import requests

import threescale_api.defaults


def assert_resource(resource: threescale_api.defaults.DefaultResource):
    assert resource is not None
    assert resource.entity_id is not None
    assert resource.entity is not None


def assert_errors_contains(resource: threescale_api.defaults.DefaultClient, fields: List[str]):
    errors = resource['errors']
    assert errors is not None
    for field in fields:
        assert field in errors


def assert_resource_params(obj: threescale_api.defaults.DefaultResource, params: dict, allowed=None):
    for (key, val) in params.items():
        if allowed is not None and key in allowed:
            assert obj[key] == val, f"Resource value for key \"{key}\" should be correct."
            assert obj.entity[key] == val, "Entity value for key \"{key}\" should be correct."


def assert_http_ok(response: requests.Response):
    assert response.status_code == 200



def test_read_by_name_account(account, api):
    """Test for read_by_name when entity has entity_name"""
    acc = api.accounts.read_by_name(account.entity_name)
    assert acc == account


def test_read_by_name_account_plan(account_plan, api):
    """Test for read_by_name when entity hasn't entity_name"""
    acc_plan = api.account_plans.read_by_name(account_plan.entity_name)
    assert acc_plan == account_plan


def test_read_by_name_application(application, account, api):
    """Test for read_by_name when entity has entity_name"""
    app = account.applications.read_by_name(application.entity_name)
    assert app == application



from threescale_api.utils import HttpClient


def test_api_client(application, proxy):
    application.api_client_verify = False
    api_client = application.api_client()

    assert api_client is not None
    assert api_client._session.verify is False
    assert api_client.get("/get").status_code == 200


def always_no_ssl_client(application, endpoint, verify):
    client = HttpClient(application, endpoint, verify)
    client._verify = False
    return client


def test_api_client_replacement(application, proxy):
    application.api_client_verify = True
    application._client_factory = always_no_ssl_client
    api_client = application.api_client()

    assert api_client is not None
    assert api_client._session.verify is False
    assert api_client.get("/get").status_code == 200



def test_policies_insert_append(proxy):
    #test_append
    policies = proxy.policies.list()
    policy_1 = {
                "name": "logging",
                "configuration": {},
                "version": "builtin",
                "enabled": True
            }
    proxy.policies.append(policy_1)
    policies["policies_config"].append(policy_1)       
    updated_policies = proxy.policies.list()
    assert policies["policies_config"] == updated_policies["policies_config"]
    #test_insert
    policy_2 = {
                "name": "echo",
                "configuration": {},
                "version": "builtin",
                "enabled": True
            }
    proxy.policies.insert(1, policy_2)
    updated_policies["policies_config"].insert(1, policy_2)
    newly_updated_policies = proxy.policies.list()
    assert updated_policies["policies_config"] == newly_updated_policies["policies_config"]


import pytest


@pytest.fixture(scope="module")
def proxy(service, proxy):
    service.proxy.update(params={"auth_user_key": "ukey"})


def test_different_user_key(proxy, application, ssl_verify):
    client = application.api_client(verify=ssl_verify)

    response = client.get("/get")
    assert response.status_code == 200
    assert "ukey" in response.request.url



from . import asserts

def test_access_tokens_list(api, access_token):
    access_tokens = api.access_tokens.list()
    assert len(access_tokens) >= 1


def test_access_token_can_be_created(api, access_token, access_token_params):
    asserts.assert_resource(access_token)
    asserts.assert_resource_params(access_token, access_token_params)


def test_access_token_can_be_read(api, access_token, access_token_params):
    read = api.access_tokens.read(access_token.entity_id)
    asserts.assert_resource(read)
    asserts.assert_resource_params(read, access_token_params)


def test_access_token_can_be_read_by_name(api, access_token, access_token_params):
    access_token_name = access_token["name"]
    read = api.access_tokens[access_token_name]
    asserts.assert_resource(read)
    asserts.assert_resource_params(read, access_token_params)



from tests.integration import asserts


def test_provider_user_can_be_created(provider_account, provider_account_params):
    asserts.assert_resource(provider_account)
    asserts.assert_resource_params(provider_account, provider_account_params)


def test_provider_account_list(api):
    accounts = api.provider_accounts.list()
    assert len(accounts) > 0


def test_provider_account_can_be_read(api, provider_account, provider_account_params):
    account = api.provider_accounts.read(provider_account.entity_id)
    asserts.assert_resource(account)
    asserts.assert_resource_params(account, provider_account_params)


def test_resource_role_change(provider_account):
    assert provider_account['role'] == 'member'
    updated = provider_account.set_role_admin()
    assert updated['role'] == 'admin'


def test_api_role_change(api, provider_account):
    assert provider_account['role'] == 'member'
    updated = api.provider_accounts.set_role_admin(provider_account.entity_id)
    assert updated['role'] == 'admin'


def test_api_read_permissions(api, provider_account):
    provider_account.set_role_admin()
    response = api.provider_accounts.permissions_read(provider_account.entity_id)
    permissions = response['permissions']
    assert 'portal' in permissions['allowed_sections']


def test_resource_read_permissions(provider_account):
    provider_account.set_role_admin()
    response = provider_account.permissions_read()
    permissions = response['permissions']
    assert 'portal' in permissions['allowed_sections']


def test_resource_update_permissions(service, provider_account):
    provider_account.set_role_member()
    response = provider_account.permissions_update()
    permissions = response['permissions']
    assert 'portal' not in permissions['allowed_sections']
    assert service['id'] not in permissions['allowed_service_ids']

    response = provider_account.permissions_update(
        allowed_services=[service['id']], allowed_sections=['portal'])
    permissions = response['permissions']
    assert 'portal' in permissions['allowed_sections']
    assert service['id'] in permissions['allowed_service_ids']



import base64

import pytest


@pytest.fixture(scope="module")
def service_params(service_params):
    service_params.update(backend_version="2")
    return service_params


@pytest.fixture(scope="module")
def proxy(service, proxy):
    service.proxy.update(params={
        "credentials_location": "authorization"
    })


def test_app_key_authorization(proxy, application, ssl_verify):
    creds = application.authobj.credentials
    encoded = base64.b64encode(
        f"{creds['app_id']}:{creds['app_key']}".encode("utf-8")).decode("utf-8")

    response = application.test_request(verify=ssl_verify)

    assert response.status_code == 200
    assert response.request.headers["Authorization"] == "Basic %s" % encoded



from tests.integration import asserts
from .asserts import assert_resource, assert_resource_params

def test_active_docs_fetch(active_doc):
    ac = active_doc.client.fetch(int(active_doc['id']))
    assert ac
    assert ac['id'] == active_doc['id']



import pytest

from tests.integration import asserts


def test_should_list_mapping_rules(backend, backend_mapping_rule):
    resource = backend.mapping_rules.list()
    assert resource

def test_should_create_mapping_rule(backend_mapping_rule, backend_mapping_rule_params):
    asserts.assert_resource(backend_mapping_rule)
    asserts.assert_resource_params(backend_mapping_rule, backend_mapping_rule_params)

def test_should_mapping_rule_endpoint_return_ok(service,
        backend_mapping_rule, backend_usage, apicast_http_client):
    service.proxy.deploy()

    response = apicast_http_client.get(path=backend_mapping_rule['pattern'])
    asserts.assert_http_ok(response)


def test_should_fields_be_required(backend, updated_backend_mapping_rules_params):
    del updated_backend_mapping_rules_params['delta']
    del updated_backend_mapping_rules_params['http_method']
    del updated_backend_mapping_rules_params['metric_id']
    resource = backend.mapping_rules.create(params=updated_backend_mapping_rules_params, throws=False)
    asserts.assert_errors_contains(resource, ['delta', 'http_method', 'metric_id'])

def test_should_read_mapping_rule(backend_mapping_rule, backend_mapping_rule_params):
    resource = backend_mapping_rule.read()
    asserts.assert_resource(resource)
    asserts.assert_resource_params(resource, backend_mapping_rule_params)


def test_should_update_mapping_rule(service,
        backend, backend_usage, updated_backend_mapping_rules_params, apicast_http_client):
    resource = backend.mapping_rules.create(updated_backend_mapping_rules_params)
    pattern = '/get/anything/test-foo'
    resource['pattern'] = pattern
    resource.update()
    updated_resource = resource.read()
    assert updated_resource['pattern'] == pattern

    service.proxy.deploy()

    response = apicast_http_client.get(path=pattern)
    asserts.assert_http_ok(response)


def test_should_delete_mapping_rule(backend, updated_backend_mapping_rules_params):
    resource = backend.mapping_rules.create(params=updated_backend_mapping_rules_params)
    assert resource.exists()
    resource.delete()
    assert not resource.exists()


def test_stop_processing_mapping_rules_once_first_one_is_met(service,
        backend_usage, backend, updated_backend_mapping_rules_params, apicast_http_client):
    params_first = updated_backend_mapping_rules_params.copy()
    params_first['pattern'] = '/get/anything/search'
    resource_first = backend.mapping_rules.create(params=params_first)
    assert resource_first.exists()

    params_second = updated_backend_mapping_rules_params.copy()
    params_second['pattern'] = '/get/anything/{id}'
    resource_second = backend.mapping_rules.create(params=params_second)
    assert resource_second.exists()

    service.proxy.deploy()

    response = apicast_http_client.get(path=params_first['pattern'])
    asserts.assert_http_ok(response)

    assert params_first['pattern'] in response.url



import logging
from logging.config import dictConfig


def _to_log_level_map(log_map: dict) -> dict:
    result = {}
    for level, arr in log_map.items():
        for item in arr:
            result[item] = level
    return result


_LOG_LEVEL_MAP_DEFINITION = {
    logging.CRITICAL: ['critical', 'c', 'crit'],
    logging.ERROR: ['error', 'e', 'err'],
    logging.WARNING: ['warning', 'w', 'warn'],
    logging.INFO: ['info', 'i', 'inf'],
    logging.DEBUG: ['debug', 'd', 'dbg'],
    logging.NOTSET: ['notset', 'n', 'nst'],
}

LOG_LEVEL_MAP = _to_log_level_map(_LOG_LEVEL_MAP_DEFINITION)


def to_log_level(level: str, default=logging.NOTSET) -> int:
    if isinstance(level, int):
        return level

    if level is not None:
        return LOG_LEVEL_MAP.get(level.lower(), default)
    return default


def load_config(level: str = "INFO", handler_level=None, api_level=None, tests_level=None):
    config = dict(
        version=1,
        formatters={
            'verbose': {
                'format':
                    '%(asctime)s %(name)-12s %(levelname)-8s %(message)s'
            }
        },
        handlers={
            'console': {
                'class': 'logging.StreamHandler',
                'formatter': 'verbose',
                'level': to_log_level(handler_level, default=level)
            }
        },
        loggers={
            'threescale_api': {
                'handlers': ['console'],
                'level': to_log_level(api_level, default=level)
            },
            'tests': {
                'handlers': ['console'],
                'level': to_log_level(handler_level, default=level)
            }
        },
    )

    dictConfig(config)
